#!/usr/bin/perl
use strict;
use warnings;
use Test::More;
use Test::Exception;

BEGIN { use_ok('SlidingWindowSearch', qw(ParsePatternSpecFile)) }

{
	SlidingWindowSearch::SetSilent(1); 
	my $silent=SlidingWindowSearch::GetSilent();
	is ($silent, 1, 'Silent mode turned on');
}	
{
	SlidingWindowSearch::SetSilent(0); 
	my $silent=SlidingWindowSearch::GetSilent();
	is ($silent, 0, 'Silent mode turned off');
}
SlidingWindowSearch::SetSilent(1);
{
	my @results = ParsePatternSpecFile('test','* Pattern width: 3','2:A');
	is_deeply(\@results,
		[0, ['.[A].'], 3, undef],
		'Simple spec file with one exact match');
}
{
	my @results = ParsePatternSpecFile('test','* Pattern width: 3','1:AB','3:PQ');
	is_deeply(\@results,
		[0, ['[AB].[PQ]'], 3, undef],
		'Simple spec file with two 2-way alternate matches');
}
{
	my @results = ParsePatternSpecFile('test','* Pattern width: 3','*Extended pattern width: 5','1:AB','3:PQ');
	is_deeply(\@results,
		[0, ['[AB].[PQ]'], 3, 5],
		'Spec file with two 2-way alternate matches and an extended width');
}
{
	my @results = ParsePatternSpecFile('test','* Pattern width: 3','1:AB','','3:PQ');
	is_deeply(\@results,
		[0, ['[AB]..','..[PQ]'], 3, undef],
		'Spec file containing two patterns');
}
{
	my @results = ParsePatternSpecFile('test',['* Pattern width: 3','1:PQ','','3:AB']);
	is_deeply(\@results,
		[0, ['[PQ]..','..[AB]'], 3, undef],
		'Spec file containing two patterns, passed as a reference');
}
{
	my @results = ParsePatternSpecFile('test','* Pattern width = 3','1 = A, B','3 = P, Q');
	is_deeply(\@results,
		[0, ['[AB].[PQ]'], 3, undef],
		'Spec file using commas and equals');
}

{	
	my ($iFail,undef) = ParsePatternSpecFile('test','1:AB');
		is $iFail, 23, 'Fail code if pattern specified before pattern width';	
}
{	
	my ($iFail,undef) = ParsePatternSpecFile('test','#');
		is $iFail, 40, 'Fail code if no patterns extracted from file';	
}
{	
	my ($iFail,undef) = ParsePatternSpecFile('test','qwertyuiop');
		is $iFail, 30, 'Fail code if file contains a nonsense line';	
}
{	
	my ($iFail,undef) = ParsePatternSpecFile('test','* Pattern width: 3','1:PQ','2.5:AB');
		is $iFail, 30, 'Fail code if line syntax is broken';	
}
{	
	my ($iFail,undef) = ParsePatternSpecFile('test','* Pattern width: 3','3:AB','1:CD');
		is $iFail, 22, 'Fail code if test positions are not strictly increasing';	
}
{	
	my ($iFail,undef) = ParsePatternSpecFile('test','* Pattern width: 3','0:AB');
		is $iFail, 21, 'Fail code if pattern test position is zero';	
}
{	
	my ($iFail,undef) = ParsePatternSpecFile('test','* Pattern width: 3','4:AB');
		is $iFail, 24, 'Fail code if pattern test position is greater than specified pattern width';	
}
{	
	my ($iFail,undef) = ParsePatternSpecFile('test','* Extended pattern width: 3','2:AB');
		is $iFail, 12, 'Fail code if extended pattern width is specified without pattern width';	
}
{	
	my ($iFail,undef) = ParsePatternSpecFile('test','* Pattern width: 4','* Extended pattern width: 3','2:AB');
		is $iFail, 13, 'Fail code if extended pattern width is specified less than pattern width';	
}
{	
	my ($iFail,undef) = ParsePatternSpecFile('test','* Pattern width: 4','1:PQ','','* Pattern width: 3','2:AB');
		is $iFail, 10, 'Fail code if attempt is made to redefine width';	
}

{
	my @results = ParsePatternSpecFile('test','* Pattern width: 3','1:PQ','','* Pattern width: 3','2:AB');
	is_deeply(\@results,
		[0, ['[PQ]..', '.[AB].'], 3, undef],
		'Okay to repeat width definition');
}
{	
	my ($iFail,undef) = ParsePatternSpecFile('test','* Pattern width: 3','* Extended pattern width: 5','1:PQ','','* Extended pattern width: 4','2:AB');
		is $iFail, 11, 'Fail code if attempt is made to redefine extended pattern width';	
}
{
	my @results = ParsePatternSpecFile('test','* Pattern width: 3','* Extended pattern width: 5','1:PQ','','* Extended pattern width: 5','2:AB');
	is_deeply(\@results,
		[0, ['[PQ]..', '.[AB].'], 3, 5],
		'Okay to repeat extended pattern width definition');
}
		
done_testing();	

